package com.muyu.common.system.remote;

import com.muyu.common.core.constant.ServiceNameConstants;
import com.muyu.common.core.domain.Result;
import com.muyu.common.system.remote.factory.RemoteDataFallbackFactory;
import lombok.NonNull;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 数据处理服务
 *
 * @Author: 胡杨
 * @Name: RemoteDataService
 * @Description: 数据处理服务
 * @CreatedDate: 2024/10/10 下午6:25
 * @FilePath: com.muyu.common.system.remote
 */

@FeignClient(contextId = "remoteDataService", value = ServiceNameConstants.DATA_SERVICE, fallbackFactory = RemoteDataFallbackFactory.class)
public interface RemoteDataService {

    /**
     * 车辆实时数据订阅开关
     * @param vin 车辆vin码
     * @param status 开关状态
     * @return 返回结果
     */
    @GetMapping("/DataProcessing/carRealTimeStatus")
    public Result<String> carRealTimeStatus(@RequestParam("vin") @NonNull String vin, @RequestParam("status") @NonNull Boolean status);

}
