package com.muyu.openbusiness.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.openbusiness.config.SnowflakeIdWorker;
import com.muyu.openbusiness.domain.WarnRule;
import com.muyu.openbusiness.domain.WarnRuleDTO;
import com.muyu.openbusiness.service.IWarnRuleService;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

/**
 * 预警规则Controller
 *
 * @author muyu
 * @date 2024-09-20
 */
@RestController
@RequestMapping("/rule")
public class WarnRuleController extends BaseController {
    @Resource
    private IWarnRuleService warnRuleService;

    @Autowired
    RedisTemplate redisTemplate;
    /**
     * 查询预警规则列表
     */
    @RequiresPermissions("warning:list")
    @GetMapping("/list")
    public Result<TableDataInfo<WarnRuleDTO>> list(WarnRuleDTO warnRuleDTO) {
        startPage();
        List<WarnRuleDTO> list = warnRuleService.selectWarnRuleList(warnRuleDTO);
//        MPJLambdaQueryWrapper<WarnRule> objectMPJLambdaQueryWrapper = new MPJLambdaQueryWrapper<>()
//                .selectAll(WarnRule.class)
//                .select(WarnStrategy::getStrategyName)
//                .leftJoin(WarnStrategy.class, WarnStrategy::getId, WarnRule::getStrategyId)
//                .stringQuery();
        SnowflakeIdWorker snowMaker = new SnowflakeIdWorker();
        for (int i = 0; i < 100; i++) {
            long ids = snowMaker.nextId();
            System.out.println(ids);

        }
        redisTemplate.opsForValue().set("ids",getDataTable(list));

        return getDataTable(list);
    }

    /**
     * 导出预警规则列表
     */
    @RequiresPermissions("warning:export")
    @PostMapping("/export")
    public void export(HttpServletResponse response, WarnRuleDTO warnRuleDTO) {
        List<WarnRuleDTO> list = warnRuleService.selectWarnRuleList(warnRuleDTO);
        ExcelUtil<WarnRuleDTO> util = new ExcelUtil<WarnRuleDTO>(WarnRuleDTO.class);

        util.exportExcel(response, list, "预警规则数据");
    }

    /**
     * 获取预警规则详细信息
     */
    @RequiresPermissions("warning:query")
    @GetMapping(value = "/{id}")
    public Result<List<WarnRule>> getInfo(@PathVariable("id") Long id) {
        return success(warnRuleService.selectWarnRuleById(id));
    }

    /**
     * 新增预警规则
     */
    @RequiresPermissions("warning:add")
    @PostMapping
    public Result<Integer> add(
            @Validated @RequestBody WarnRuleDTO warnRuleDTO) {
        if (warnRuleService.checkIdUnique(warnRuleDTO)) {
            return error("新增 预警规则 '" + warnRuleDTO + "'失败，预警规则已存在");
        }
        return toAjax(warnRuleService.save(warnRuleDTO));
    }

    /**
     * 修改预警规则
     */
    @RequiresPermissions("warning:edit")
    @PutMapping
    public Result<Integer> edit(
            @Validated @RequestBody WarnRuleDTO warnRuleDTO) {
        if (!warnRuleService.checkIdUnique(warnRuleDTO)) {
            return error("修改 预警规则 '" + warnRuleDTO + "'失败，预警规则不存在");
        }
        return toAjax(warnRuleService.updateById(warnRuleDTO));
    }

    /**
     * 删除预警规则
     */
    @RequiresPermissions("warning:remove")
    @DeleteMapping("/{ids}")
    public Result<Integer> remove(@PathVariable("ids") Long[] ids) {
        warnRuleService.removeBatchByIds(Arrays.asList(ids));
        return success();
    }

}
