package com.muyu.openbusiness.service.impl;


import javax.annotation.Resource;

import com.muyu.common.redis.service.RedisService;
import com.muyu.openbusiness.domain.SysCarData;
import com.muyu.openbusiness.service.SysCarRealTimeService;
import org.springframework.stereotype.Service;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 车辆实时信息实现层
 *
 * @Author: 胡杨
 * @Name: SysCarRealTime
 * @Description: 车辆实时信息实现层
 * @CreatedDate: 2024/10/10 下午7:37
 * @FilePath: com.muyu.openbusiness.service.impl
 */

@Slf4j
@Service
public class SysCarRealTimeServiceImpl implements SysCarRealTimeService {
    @Resource
    private RedisService redisService;

    private static final HashMap<String, Class<?> > typeMap = new HashMap<>();
    static {
        typeMap.put("int", Integer.class);
        typeMap.put("double", Double.class);
        typeMap.put("long", Long.class);
        typeMap.put("string", String.class);
    }

    @Override
    public HashMap<String, String> getCarRealTime(String vin) {
        HashMap<String, String> map = new HashMap<>();
        for (String key : redisService.keys(vin+":*")) {
            map.put(key.replace(vin + ":",""), classConversion(redisService.getCacheObject(key)));
        }
        return map;
    }

    /**
     * 去除无用字符
     * @param str
     * @return
     */
    private String classConversion(String str) {
        try {
            double d = Double.parseDouble(str);
            str = Double.toString(d);
            String[] split = str.split(".");
            if (split.length>1 && split[1].equals("0")) {
                str = String.valueOf(Integer.parseInt(str));
            }
            return str;
        } catch (NumberFormatException e) {
            return str;
        }



    }
}
